import * as Cesium from 'cesium';
import { CCM } from 'tomcloudmap';

class CScene {

    constructor(sceneJson = false, log = false) {
        if (sceneJson) {

            this.input = sceneJson;
            if (typeof sceneJson.cesiumIonToken != "undefined")
                Cesium.Ion.defaultAccessToken = 'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJqdGkiOiJlZjNiZTgxOS0xZDYwLTQzNzctYWRkOS00ZjJkZDI2YjA5MGMiLCJpZCI6ODMyOTksImlhdCI6MTY0NTY3MTU4NH0.2bu4bjqgk1yx5JMdC1iU8j65IlMztD4KI11scmH_sHQ';

            this.viewer = new Cesium.Viewer(sceneJson.div, {
                terrainProvider: Cesium.createWorldTerrain(),
                imageryProvider: new Cesium.UrlTemplateImageryProvider({
                    url: "https://webst02.is.autonavi.com/appmaptile?style=6&x={x}&y={y}&z={z}",//image map
                    // url: sceneJson.imagery.url,
                }),
            });
            // 这里解决锯齿
            this.viewer.scene.fxaa = false;
            this.viewer.scene.postProcessStages.fxaa.enabled = true;
            if (Cesium.FeatureDetection.supportsImageRenderingPixelated()) { // 判断是否支持图像渲染像素化处理
                this.viewer.resolutionScale = window.devicePixelRatio;
            }
        }
        else {
            this.viewer = false;
            this.log(
                {
                    type: "error",
                    subject: "sceneJson value is Null",
                    sender: "CScene constructor "
                }
            );
        }

        this.logRouter = log;

    }
    flyTo(location, toRadians = 0, radians = -65.0) {
        this.viewer.camera.flyTo({
            destination: Cesium.Cartesian3.fromDegrees(location[0], location[1], location[2]),
            orientation: {
                heading: Cesium.Math.toRadians(toRadians),
                pitch: Cesium.Math.toRadians(radians),
            }
        });
    }
    log(msg, that = false) {
        if (that == false) {
            that = this;
        }
        if (that.logRouter !== false) {
            that.inputJson.logRouter(msg);
        } else {
            console.log("CPlanet log  : ", msg);
        }
    }
}

class DcCube {

    constructor(input) {
        this.visible = true;
        this.input = input;
        this.origin = Cesium.Cartesian3.fromDegrees(input.coordinate[0], input.coordinate[1], input.coordinate[2]);
        this._modelMatrix = Cesium.Transforms.eastNorthUpToFixedFrame(this.origin);

        // this.modelMatrix = modelMatrix || Cesium.Matrix4.IDENTITY.clone()
        // this.drawCommand = null;
        this.listCommands = [];
        this.updateOfListCommands = true;
        this.color=input.color;
        this.width=input.width;
        this.init();
    }

    init(){
        
    }
    /**
     * 创建 DrawCommand
     * @param {Cesium.Context} context
     */
    createCommand(context) {

        var modelMatrix = this._modelMatrix;

        var box = new Cesium.BoxGeometry({
            vertexFormat: Cesium.VertexFormat.POSITION_ONLY,
            maximum: new Cesium.Cartesian3(this.width, this.width, this.width),
            minimum: new Cesium.Cartesian3(0.-this.width, 0.- this.width, 0-this.width)
            // maximum: new Cesium.Cartesian3(250.0, 250.0, 250.0),
            // minimum: new Cesium.Cartesian3(-250.0, -250.0, -250.0)
        });
        var geometry = Cesium.BoxGeometry.createGeometry(box);

        var attributeLocations = Cesium.GeometryPipeline.createAttributeLocations(geometry);

        var va = Cesium.VertexArray.fromGeometry({
            context: context,
            geometry: geometry,
            attributeLocations: attributeLocations
        });

        var vs = `
        attribute vec3 position;
        void main(){
            gl_Position = czm_projection  * czm_modelView * vec4( position , 1. );
        }
        `;
        var fs = `
        uniform vec3 color;
        void main(){
            gl_FragColor=vec4( color , 1. );
        }
        `;
        var shaderProgram = Cesium.ShaderProgram.fromCache({
            context: context,
            vertexShaderSource: vs,
            fragmentShaderSource: fs,
            attributeLocations: attributeLocations
        });

        var uniformMap = {
            color() {
                return Cesium.Color.GREEN
                // return Cesium.Color(this.color.r,this.color.g,this.color.b);
            }
        };

        var renderState = Cesium.RenderState.fromCache({
            cull: {
                enabled: true,
                face: Cesium.CullFace.BACK
            },
            depthTest: {
                enabled: true
            }
        });

        this.drawCommand = new Cesium.DrawCommand({
            modelMatrix: modelMatrix,
            vertexArray: va,
            shaderProgram: shaderProgram,
            uniformMap: uniformMap,
            renderState: renderState,
            pass: Cesium.Pass.OPAQUE
        });
    }

    /**
     * 实现Primitive接口，供Cesium内部在每一帧中调用
     * @param {Cesium.FrameState} frameState
     */
    update(frameState) {
        if (this.visible) {
            if (!this.drawCommand) {
                this.createCommand(frameState.context);
            }
            frameState.commandList.push(this.drawCommand);
            // let abc = frameState.commandList;
        }
    }

}

// import * as Cesium from 'cesium';

class Locations {

    constructor(parent = false, inputJson = false, log = false) {
        this.viewer = parent.scene.viewer;
        this.parent = parent;
        this.input = inputJson;
        this.list = {};
        this.init(this.input.home);
        return this;
    }
    log(msg, that = false) {
        if (that == false) {
            that = this;
        }
        if (typeof that.input.log !== "undefined") {
            that.inputJson.log(msg);
        } else {
            console.log("CPlanet log  : ", msg);
        }
    }
    init() {
        this.list['home'] = this.initOne(this.input, 'home');
    }
    initByName(name = false) {
        if (name  && typeof this.input[name] != "undefined") {
            if (typeof this.list[name] != "undefined") {
                this.parent.flyTo(this.input[name].attribute.coordinate);
            }
            else
                this.list[name] = this.initOne(this.input, name);
        }
        else {
            this.log(
                {
                    type: 'error',
                    subject: "initByName (" + name + ")" + ",name did't find.",
                    sender: "CLoations initByName()",
                    content: "value  error",
                    info: this.input
                }
            );
        }
    }
    initOne(mainJson, mainLocation, deep = 1) {
        let list = {};
        let coordinate = [];
        let toRadians = false;
        let radians = false;
        for (let i in mainJson) {
            let perLocation = mainJson[i];

            if (i == mainLocation) {
                if (deep == 1) {
                    coordinate = perLocation.attribute.coordinate;
                    if (typeof perLocation.attribute.toRadians != "undefined") {
                        toRadians = perLocation.attribute.toRadians;
                    }
                    if (typeof perLocation.attribute.radians != "undefined") {
                        radians = perLocation.attribute.radians;
                    }
                }
                if (perLocation.node === true) {
                    list[i] = {};
                    for (let j in perLocation.children) {
                        let subOne = perLocation.children[j];
                        if (subOne.node === true)
                            list[i][j] = this.initOne(subOne, j, deep + 1);
                        else if (subOne.node === false) {
                            ////////////////////////////////////////////////////////////
                            // core 
                            if (subOne.type == "CCAE" && subOne.enable === true) {

                                list[i][j] = new CCM(subOne.attribute);
                                list[i][j].visible = subOne.visible;
                                list[i][j].id = subOne.id;
                                list[i][j].name = subOne.name;

                                this.viewer.scene.globe.depthTestAgainstTerrain = true;
                                this.viewer.scene.primitives.add(list[i][j]);
                            }
                            else if (subOne.type == "DcCube" && subOne.enable === true) {
                                list[i][j] = new DcCube(subOne.attribute);
                                list[i][j].visible = subOne.visible;
                                list[i][j].id = subOne.id;
                                list[i][j].name = subOne.name;

                                this.viewer.scene.globe.depthTestAgainstTerrain = true;
                                this.viewer.scene.primitives.add(list[i][j]);
                            }
                            ///////////////////////////////////////////////////////////////
                        }
                    }
                }

                else {
                    this.log(
                        {
                            type: 'error',
                            subject: "init One: " + mainLocation,
                            sender: "CLoations initOne()",
                            content: "node error",
                            info: perLocation
                        }
                    );
                }
            }
        }
        if (deep == 1) {
            if (radians && toRadians)
                this.parent.flyTo(coordinate, toRadians, radians);
            else
                this.parent.flyTo(coordinate);
        }
        return list[mainLocation];
    }
}

class CPlanet {

    constructor(inputJson = false) {
        this.locations=false;
        if (inputJson) {
            this.input = inputJson;
            this.init(inputJson);
        }
        else {
            this.log(
                {
                    type: "error",
                    subject: "input value is Null",
                    sender: "CPlanet constructor "
                }
            );
        }

    }
    log(msg, that = false) {
        if (that == false) {
            that = this;
        }
        if (typeof that.inputJson.log !== "undefined") {
            that.inputJson.log(msg);
        } else {
            console.log("CPlanet log  : ", msg);
        }
    }
    init(input) {
        this.scene = new CScene(input.scene, this.log);
        this.locations = new Locations(this, input.locations, this.log);
    }
    getAllObjectsOfLocations() {
        return this.locations
    }

    getObjectsOfByLocationsName(name=false) {
        if(name){
            if(typeof this.locations.list != "undefined" &&  this.locations.list !==  false && typeof this.locations.list[name] != "undefined")
            return this.locations.list[name];
        }
        return false;
    }
    getObjectsOfByName() {

    }
    getObjectsOfByID() {

    }
    initLayer() {

    }

    initObjects() {

    }

    flyTo(location, toRadians = 0, radians = -65.0) {
        this.scene.flyTo(location,toRadians,radians);
    }
    afterInit(scope = false) {
        if (scope == false) {
            scope = this;
        }
    }
}

export { CPlanet };
